@@ -50,6 +50,9 @@
 #include "cfg.h"
 #include "pcre.h"
 
+/* pcre static vector size */
+#define OVECSIZE 30
+
 #ifndef LOG_PRIMASK
 #define LOG_PRIMASK (LOG_EMERG|LOG_ALERT|LOG_CRIT|LOG_ERR|LOG_WARNING|LOG_NOTICE|LOG_INFO|LOG_DEBUG)
 #endif
@@ -173,13 +176,17 @@ static struct {
     l2_channel_t *l2_nch;
     levelmap_t   *levelmap;
     int           maskpri;
+    int           logopt;
+    int           delayopen;
 } ctx = { 
     NULL,
     NULL,
     NULL,
     NULL,
     NULL,
-    LOG_PRIMASK
+    LOG_PRIMASK,
+    0,
+    FALSE
 };
 
 static void fsldebug(l2_level_t level, const char *message, ...)
@@ -292,7 +299,7 @@ void openlog(const char *ident, int logopt, int facility)
     cfg_node_type_t  cfgtype;
     int              cfgnumc;
     int              cfgnume;
-    char *cfgargtoka[3]; /*FIXME hardcoded maximum number of arguments */
+    char *cfgargtoka[3];
     char *cfgargtok;
     char *argident;
     char *argmatch;
@@ -303,7 +310,6 @@ void openlog(const char *ident, int logopt, int facility)
     char *cpIdent;
     char *cpFacility;
     char *cpISF;
-#define OVECSIZE 30 /*FIXME find a good place for this*/
     int ovec[OVECSIZE];
     const char   **acpMatch;
     l2_channel_t *ch; /* scratch variable */
@@ -319,8 +325,6 @@ void openlog(const char *ident, int logopt, int facility)
     /* properly handle repeated execution */
     closelog();
 
-    /* info: ident, facility; trace: configmatching; debug: traversal */
-
     /* create L2 environment for fsl itself */
     argl2spec = getenv("FSL_DEBUG");
     if (argl2spec == NULL)
@@ -347,8 +351,14 @@ void openlog(const char *ident, int logopt, int facility)
         fsldebug(L2_LEVEL_TRACE, "captured openlog(ident=\"%s\", logopt=0x%.8lx, facility=0x%.8lx)", ident, logopt, facility);
     }
 
-    /*FIXME currently we completely ignore logopt LOG_CONS, LOG_NDELAY, LOG_PERROR, LOG_PID
-    */
+    /* remember logopt and handle unsupported values */
+    ctx.logopt = logopt;
+    if (logopt & LOG_CONS)
+        fsldebug(L2_LEVEL_WARNING, "ignored unsupported logopt LOG_CONS");
+    if (logopt & LOG_PERROR)
+        fsldebug(L2_LEVEL_WARNING, "ignored unsupported logopt LOG_PERROR, use l2spec channel \"fd(filedescriptor=2)\" to emulate");
+    if (logopt & LOG_PID)
+        fsldebug(L2_LEVEL_WARNING, "ignored unsupported logopt PID, use l2spec formatter %%P in prefix channel to emulate");
 
     /* create default sysloglevel to l2_level mapping */
     fsldebug(L2_LEVEL_DEBUG, "create default sysloglevel to l2_level mapping\n");
@@ -363,19 +373,21 @@ void openlog(const char *ident, int logopt, int facility)
 
     /* create L2 environment for main application */
     if ((l2rv = l2_env_create(&ctx.l2_env)) != L2_OK) {
-        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "failed to create L2 environment (%d)\n", l2rv); CU(1); }
+        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "failed to create L2 environment: (%d)\n", l2rv); CU(1); }
     if ((l2rv = l2_env_levels(ctx.l2_env, L2_LEVEL_ALL, L2_LEVEL_NONE)) != L2_OK) {
-        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "logging failed to set global logging level defaults %s(%d)\n", cp, l2rv); CU(1); }
+        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "logging failed to set global logging level defaults: %s(%d)\n", cp, l2rv); CU(1); }
     if ((l2rv = l2_env_formatter(ctx.l2_env, 'D', l2_util_fmt_dump, NULL)) != L2_OK) {
-        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "logging failed to register dump formatter %s(%d)\n", cp, l2rv); CU(1); }
+        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "logging failed to register dump formatter: %s(%d)\n", cp, l2rv); CU(1); }
     if ((l2rv = l2_env_formatter(ctx.l2_env, 'S', l2_util_fmt_string, NULL)) != L2_OK) {
-        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "logging failed to register string formatter %s(%d)\n", cp, l2rv); CU(1); }
+        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "logging failed to register string formatter: %s(%d)\n", cp, l2rv); CU(1); }
     if ((l2rv = l2_env_formatter(ctx.l2_env, 'm', l2_util_fmt_errno, NULL)) != L2_OK) {
-        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "logging failed to register errno formatter %s(%d)\n", cp, l2rv); CU(1); }
-    if ((l2rv = l2_env_settimer(ctx.l2_env, 20)) != L2_OK) { /*FIXME do we need this?*/
-        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "failed to configure timer %s(%d)\n", cp, l2rv); CU(1); }
+        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "logging failed to register errno formatter: %s(%d)\n", cp, l2rv); CU(1); }
+#if 0
+    if ((l2rv = l2_env_settimer(ctx.l2_env, 20)) != L2_OK) { /*FIXME do we need this? -- RSE: no! */
+        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "failed to configure timer: %s(%d)\n", cp, l2rv); CU(1); }
+#endif
     if ((l2rv = l2_channel_create(&ctx.l2_nch, ctx.l2_env, "noop")) != L2_OK) {
-        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "logging failed to create noop channel; %s(%d)\n", cp, l2rv); CU(1); }
+        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "logging failed to create noop channel: %s(%d)\n", cp, l2rv); CU(1); }
 
     /* create IdentSlashFacility */
     if ((cpIdent = strdup((ident != NULL) ? ident : "unknown")) == NULL) {
@@ -495,7 +507,13 @@ void openlog(const char *ident, int logopt, int facility)
                     fsldebug(L2_LEVEL_ERROR, "pcre_study() failed with error %s\n", cpError); CU(1); }
 
                 nMatch = pcre_exec(pcreRegex, pcreExtra, cpISF, strlen(cpISF), 0, 0, ovec, OVECSIZE);
-                fsldebug(L2_LEVEL_TRACE, "found %2d matches when \"%s\" is used against \"%s\"\n", nMatch, argmatch, cpISF);
+                if (nMatch < 0)
+                    fsldebug(L2_LEVEL_TRACE, "matching ident/facility \"%s\" against section \"%s\" failed.\n", cpISF, argmatch);
+                else
+                    if (nMatch == 0)
+                        fsldebug(L2_LEVEL_TRACE, "matching ident/facility \"%s\" against section \"%s\" succeeded, found $0\n", cpISF, argmatch);
+                    else
+                        fsldebug(L2_LEVEL_TRACE, "matching ident/facility \"%s\" against section \"%s\" succeeded, found $0...$%d\n", cpISF, argmatch, (nMatch-1) > 9 ? 9 : (nMatch-1));
                 if (nMatch >= 1) {
                     pcre_get_substring_list(cpISF, ovec, nMatch, &acpMatch);
                     if (acpMatch != NULL)
@@ -548,10 +566,15 @@ void openlog(const char *ident, int logopt, int facility)
         if ((cfgrv = cfg_node_get(cfg, cfgdir, CFG_NODE_ATTR_RBROTH, &cfgdir)) != CFG_OK) {
             (void)cfg_error(cfg, cfgrv, &cp); fsldebug(L2_LEVEL_ERROR, "cfg_node_get(CFG_NODE_ATTR_RBROTH) failed with error %s (%d)\n", cp, cfgrv); CU(1); }
     }
-    if ((l2rv = l2_channel_open(ctx.l2_nch)) != L2_OK) {
-        cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "logging failed to open channel stream %s(%d)\n", cp, l2rv); CU(1); }
-
-    ctx.maskpri = LOG_UPTO(LOG_DEBUG);
+    if (logopt & LOG_NDELAY) {
+        ctx.delayopen = TRUE;
+        fsldebug(L2_LEVEL_TRACE, "logopt LOG_NDELAY delays open of L2 channel tree until first message is being logged");
+    }
+    else {
+        if ((l2rv = l2_channel_open(ctx.l2_nch)) != L2_OK) {
+            cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "logging failed to open channel stream %s(%d) immediately\n", cp, l2rv); CU(1); }
+        fsldebug(L2_LEVEL_TRACE, "logging succeeded to open channel stream immediately");
+    }
 
     CU(0);
 CUS:
@@ -594,6 +617,9 @@ void closelog(void)
         free(ctx.levelmap);
         ctx.levelmap = NULL;
     }
+    ctx.maskpri = LOG_PRIMASK;
+    ctx.logopt = 0;
+    ctx.delayopen = FALSE;
 }
 
 int setlogmask(int maskpri)
@@ -631,11 +657,21 @@ void vsyslog(int priority, const char *fmt, va_list args)
     int i;
     l2_result_t l2rv;
     char *cp;
-
+    
     /* check for previous proper initialization */
     if (ctx.l2_nch == NULL)
         return;
 
+    /* check for delayed open */
+    if ((ctx.logopt & LOG_NDELAY) && (ctx.delayopen == TRUE)) {
+        if ((l2rv = l2_channel_open(ctx.l2_nch)) != L2_OK) {
+            cp = l2_env_strerror(ctx.l2_env, l2rv); fsldebug(L2_LEVEL_ERROR, "logging failed to open channel stream %s(%d) delayed\n", cp, l2rv);
+            closelog();
+        }
+        fsldebug(L2_LEVEL_TRACE, "logging succeeded to open channel stream delayed");
+        ctx.delayopen = FALSE;
+    }
+
     /* strip off facility */
     priority &= LOG_PRIMASK;
     fsldebug(L2_LEVEL_DEBUG, "prioriy  =0x%.8lx, ctx.maskpri=0x%.8lx  ", (unsigned long)priority, (unsigned long)ctx.maskpri);
